<?php

namespace Drupal\abstract_email_validation\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Abstract email validation settings for this site.
 */
final class AbstractEmailValidationSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'abstract_email_validation_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['abstract_email_validation.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $config = $this->config('abstract_email_validation.settings');

    $form['abstract'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Abstract API'),
      '#description' => $this->t('Get your base url and api key from <a href="https://www.abstractapi.com/api/email-verification-validation-api" target="_blank">here</a>.'),
    ];

    $form['abstract']['api_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base Url'),
      '#default_value' => $config->get('api_url'),
      '#required' => TRUE,
    ];

    $form['abstract']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
    ];

    $form['custom_error_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom Error Message'),
      '#size' => 100,
      '#description' => $this->t('The dynamic error message. Where %email will be replaced by the user email address.'),
      '#default_value' => $config->get('custom_error_message'),
    ];

    $form['user_registration_form_email_validation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the email validation on register form.'),
      '#description' => $this->t('Validate the email using the Abstract Email Validation API.'),
      '#default_value' => $config->get('user_registration_form_email_validation'),
    ];

    $form['detailed_settings'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the Detailed Email Validation'),
      '#description' => $this->t('<b> We are validating the email based on the following details
        <br> 1. quality_score >= 0.8
        <br> 2. is_smtp_valid = TRUE
        <br> 3. deliverability = DELIVERABLE
        <br> Please specify the conditions for mail validation (choose from the following options).
        </b>'),
      '#default_value' => $config->get('detailed_settings.enabled'),
    ];

    $form['details'] = [
      '#type' => 'details',
      '#title' => 'Detailed Settings',
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="detailed_settings"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['details']['smtp'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the email validation based on SMTP.'),
      '#description' => $this->t('Validate using the value of "is_smtp_valid" key of the API response.'),
      '#default_value' => $config->get('detailed_settings.smtp'),
    ];
    $form['details']['mx_found'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the email validation based on MX_FOUND.'),
      '#description' => $this->t('Validate using the value of "mx_found" key of the API response.'),
      '#default_value' => $config->get('detailed_settings.mx_found'),
    ];
    $form['details']['quality_score'] = [
      '#type' => 'number',
      '#title' => $this->t('Enter the desired QUALITY_SCORE of email.'),
      '#description' => $this->t('Validate using the value of "quality_score" key of the API response. <br> It varies between 0.0 to 1.0. Recommended the "quality_score" is >= 0.8'),
      '#default_value' => $config->get('detailed_settings.quality_score'),
      '#min' => 0.1,
      '#max' => 1.0,
      '#step' => 0.1,
    ];
    $form['details']['deliverability'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable the email validation based on Deliverability.'),
      '#description' => $this->t('Validate using the value of "deliverability" key of the API response.'),
      '#default_value' => $config->get('detailed_settings.deliverability'),
    ];

    $form['blacklist'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Blacklist'),
    ];
    $form['blacklist']['blacklist_email_option'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Block email address(es) based on ids.'),
      '#default_value' => $config->get('blacklist.blacklist_email_option'),
    ];
    $form['blacklist']['emails'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Add emails to block.'),
      '#default_value' => $config->get('blacklist.emails'),
      '#states' => [
        'visible' => [
          ':input[name="blacklist_email_option"]' => ['checked' => TRUE],
        ],
      ],
      '#description' => $this->t('Please use comma(,) for entering multiples emails Ex (example@example.com, example2@example.com)'),
    ];

    $form['blacklist']['blacklist_domain_option'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Block email address based on domains.'),
      '#default_value' => $config->get('blacklist.blacklist_domain_option'),
    ];
    $form['blacklist']['domains'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Add domains to block.'),
      '#default_value' => $config->get('blacklist.domains'),
      '#states' => [
        'visible' => [
          ':input[name="blacklist_domain_option"]' => ['checked' => TRUE],
        ],
      ],
      '#description' => $this->t('Please use comma(,) for entering multiples domain Ex (example.com, example2.com)'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $emails = $form_state->getValue('emails');
    $domains = $form_state->getValue('domains');

    if ($form_state->getValue('detailed_settings')) {
      $details_fields = [
        'smtp',
        'mx_found',
        'quality_score',
        'deliverability',
      ];
      $one_checked = FALSE;

      // We are making sure that at least one of the details fields is selected.
      foreach ($details_fields as $field) {
        if ($form_state->getValue($field)) {
          $one_checked = TRUE;
          break;
        }
      }

      if (!$one_checked) {
        $form_state->setErrorByName('detailed_settings', $this->t('You must check at least one of the detailed settings fields if "Enable the Detailed Email Validation" is selected.'));
      }
    }

    if ($form_state->getValue('blacklist_email_option') && !empty($emails)) {
      $blacklisted_emails = $this->sanitize($emails);
      foreach ($blacklisted_emails as $email) {

        // Check the email address format.
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
          $form_state->setErrorByName(
            'emails',
            $this->t('Email address i.e @email is incorrect', ['@email' => $email])
          );
        }
      }
    }

    if ($form_state->getValue('blacklist_domain_option') && !empty($domains)) {
      $blacklisted_domains = $this->sanitize($domains);
      foreach ($blacklisted_domains as $domain) {

        // Check the email address format.
        if (!filter_var($domain, FILTER_VALIDATE_DOMAIN)) {
          $form_state->setErrorByName(
            'domains',
            $this->t('Domain i.e @domain is incorrect', ['@domain' => $domain])
          );
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('abstract_email_validation.settings');
    $config->clear('api_key');
    $config->clear('api_url');
    $config->clear('blacklist');
    $config->clear('detailed_settings');

    $config->set('api_url', $form_state->getValue('api_url'));
    $config->set('api_key', $form_state->getValue('api_key'));
    $config->set('user_registration_form_email_validation', $form_state->getValue('user_registration_form_email_validation'));

    $config->set('detailed_settings.enabled', $form_state->getValue('detailed_settings'));
    $config->set('detailed_settings.smtp', $form_state->getValue('smtp'));
    $config->set('detailed_settings.mx_found', $form_state->getValue('mx_found'));
    $config->set('detailed_settings.quality_score', $form_state->getValue('quality_score'));
    $config->set('detailed_settings.deliverability', $form_state->getValue('deliverability'));

    $config->set('custom_error_message', $form_state->getValue('custom_error_message'));

    $config->set('blacklist.blacklist_email_option', $form_state->getValue('blacklist_email_option'));
    $config->set('blacklist.emails', $form_state->getValue('emails'));
    $config->set('blacklist.blacklist_domain_option', $form_state->getValue('blacklist_domain_option'));
    $config->set('blacklist.domains', $form_state->getValue('domains'));

    $config->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * It takes a string of emails or domains and make an array of it.
   *
   * @param string $data
   *   This can be emails or domains.
   *
   * @return array
   *   Return a blacklisted emails or domains array.
   */
  public function sanitize(string $data) {
    $data_array = explode(',', $data);
    foreach ($data_array as $value) {
      $blacklisted_emails[$value] = trim(strtolower($value));
    }
    return $blacklisted_emails;
  }

}
