<?php

namespace Drupal\abstract_email_validation\Http;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Service for requesting Abstract API.
 */
class AbstractApi {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * Constructs the AbstractApi.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger service.
   */
  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('abstract_email_validation');
  }

  /**
   * Request the Abstract API and send the response.
   *
   * @param string $email
   *   The email address to validate.
   *
   * @return array
   *   TRUE if the email is valid, FALSE otherwise.
   */
  public function callAbstractApi(string $email) {
    $result = [];
    $config = $this->configFactory->getEditable('abstract_email_validation.settings');
    $api_base_url = $config->get('api_url');
    $api_key = $config->get('api_key');
    try {
      $response = $this->httpClient->request('GET', $api_base_url, [
        'query' => [
          'api_key' => $api_key,
          'email' => $email,
        ],
      ]);
      $result['status_code'] = $response->getStatusCode();
      $data = JSON::decode($response->getBody());
      $result['data'] = $data;
    }
    catch (\Exception $e) {
      $result['status_code'] = $e->getCode();
      $this->logger->error('API Request failed: @message', ['@message' => $e->getMessage()]);
    }
    return $result;
  }

}
