<?php

namespace Drupal\abstract_email_validation\Plugin\Validation\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Drupal\abstract_email_validation\Validate\EmailValidator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Validates the AbstractEmailValidation constraint.
 */
class AbstractEmailValidationConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The Abstract Api service.
   *
   * @var \Drupal\abstract_email_validation\Validate\EmailValidator
   */
  protected EmailValidator $emailValidatorService;

  /**
   * The Config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new AbstractEmailValidationConstraintValidator.
   *
   * @param \Drupal\abstract_email_validation\Validate\EmailValidator $email_validation_service
   *   The email validator service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   */
  public function __construct(EmailValidator $email_validation_service, ConfigFactoryInterface $config_factory) {
    $this->emailValidatorService = $email_validation_service;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('abstract_email_validation.validator'),
      $container->get('config.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    $config = $this->configFactory->get('abstract_email_validation.settings');
    $custom_error_message = $config->get('custom_error_message');
    if (empty($value)) {
      return;
    }
    $field_values = $value->getValue();
    foreach ($field_values as $field_value) {
      $validation_result = $this->emailValidatorService->validate($field_value['value']);

      if (!$validation_result['status']) {
        if (isset($validation_result['message'])) {
          $this->context->addViolation($validation_result['message']);
        }
        else {
          // phpcs:ignore Drupal.Semantics.FunctionT.NotLiteralString
          $this->context->addViolation($this->t($custom_error_message, [
            '%email' => $field_value['value'],
          ]));
        }
      }
    }
  }

}
