<?php

namespace Drupal\abstract_email_validation\Validate;

use Drupal\abstract_email_validation\Http\AbstractApi;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides the services to validate email ID.
 *
 * This class contains different functions for validation based on
 * Abstract API response and the different settings set by the admin user.
 */
class EmailValidator {

  use StringTranslationTrait;

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * Abstract Api Handler.
   *
   * @var \Drupal\abstract_email_validation\Http\AbstractApi
   */
  protected AbstractApi $abstractApiHandler;

  /**
   * UserSettings constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   * @param \Drupal\abstract_email_validation\Http\AbstractApi $abstract_api_handler
   *   Abstract Api Handler.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger,
    AbstractApi $abstract_api_handler,
    ) {
    $this->configFactory = $config_factory;
    $this->logger = $logger->get('abstract_email_validation');
    $this->abstractApiHandler = $abstract_api_handler;
  }

  /**
   * Validate the email Id based on the different settings and api response.
   *
   * @param string $email
   *   The email ID.
   *
   * @return array
   *   return the validation result as an array.
   */
  public function validate(string $email) {
    $config = $this->configFactory->getEditable('abstract_email_validation.settings');
    $detailed_settings = $config->get('detailed_settings');
    $validation_result = [];
    if ($config->get('blacklist.blacklist_email_option') &&
      !empty($config->get('blacklist.emails')) &&
      $this->checkBlacklistEmail($email)) {

      $validation_result['status'] = FALSE;
      $validation_result['message'] = $this->t('This email is banned for the site. Please Contact site admin.');
      return $validation_result;
    }
    if ($config->get('blacklist.blacklist_domain_option') &&
      !empty($config->get('blacklist.domains')) &&
      $this->checkBlacklistDomain($email)) {

      $validation_result['status'] = FALSE;
      $validation_result['message'] = $this->t('This domain is banned for the site. Please Contact site admin.');
      return $validation_result;
    }

    $result = $this->abstractApiHandler->callAbstractApi($email);
    if ($result['status_code'] == 200) {
      $validation_result['status'] = $this->detailedValidation($detailed_settings, $result['data']);
    }
    else {
      $validation_result['status'] = FALSE;
      $validation_result['message'] = $this->t('Unable to verify email. Please Contact site admin.');
    }
    return $validation_result;
  }

  /**
   * Check the email address based on configuration.
   *
   * @param array $detailed_settings
   *   Configuration keys.
   * @param array $data
   *   Data array getting from API response.
   *
   * @return bool
   *   Return TRUE if it meets the configured requirements, otherwise FALSE.
   */
  public function detailedValidation(array $detailed_settings, array $data) {
    if ($detailed_settings['enabled']) {
      $response = TRUE;
      if ($detailed_settings['smtp']) {
        $response = $response && $this->getSmtpStatus($data);
      }
      if ($detailed_settings['mx_found']) {
        $response = $response && $this->getMxFoundStatus($data);
      }
      if ($detailed_settings['deliverability']) {
        $response = $response && $this->getDeliverabilityStatus($data);
      }
      if ($detailed_settings['quality_score']) {
        $response = $response && ($this->getQualityScoreStatus($data, $detailed_settings['quality_score']));
      }
      return $response;
    }
    // We are validating the email based on the following details.
    // 1. quality_score >= 0.8
    // 2. is_smtp_valid = TRUE.
    // 3. deliverability = DELIVERABLE.
    if ($this->getSmtpStatus($data) &&
      $this->getDeliverabilityStatus($data) &&
      $this->getQualityScoreStatus($data, 0.8)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Check is_smtp_valid.
   *
   * @param array $response
   *   The api response.
   *
   * @return bool
   *   Return the value of 'is_smtp_valid' of API response.
   */
  public function getSmtpStatus(array $response) {
    return $response['is_smtp_valid']['value'];
  }

  /**
   * Check is_mx_found.
   *
   * @param array $response
   *   The api response.
   *
   * @return bool
   *   Return the value of 'is_mx_found' of API response.
   */
  public function getMxFoundStatus(array $response) {
    return $response['is_mx_found']['value'];
  }

  /**
   * Check quality_score.
   *
   * @param array $response
   *   The api response.
   * @param float $defined_score
   *   The already defined score.
   *
   * @return bool
   *   Return TRUE if the score is greater or equals to the defined score else
   *   FALSE.
   */
  public function getQualityScoreStatus(array $response, float $defined_score) {
    $quailty_score = $response['quality_score'];
    if ($quailty_score >= $defined_score) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Check deliverability.
   *
   * @param array $response
   *   The api response.
   *
   * @return bool
   *   Return the value of 'deliverability' of API response.
   */
  public function getDeliverabilityStatus(array $response) {
    return $response['deliverability'];
  }

  /**
   * Check the provided email is present on blacklisted emails list or not.
   *
   * @param string $email
   *   Ihe user enter email address.
   *
   * @return bool
   *   Email address if present in the blaklisted array then return TRUE,
   *   else FALSE.
   */
  public function checkBlacklistEmail(string $email) {
    // Get the module configuration.
    $config = $this->configFactory->getEditable('abstract_email_validation.settings');
    $blacklist_emails = explode(',', $config->get('blacklist.emails'));
    foreach ($blacklist_emails as $mail) {
      if (trim(strtolower($email)) == trim(strtolower($mail))) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Check the provided domain is present on blacklisted domain list or not.
   *
   * @param string $email
   *   Ihe user enter email address.
   *
   * @return bool
   *   Domain if present in the blacklisted array then return TRUE, else FALSE.
   */
  public function checkBlacklistDomain(string $email) {
    $email_domain = $this->extractDomain($email);
    $config = $this->configFactory->getEditable('abstract_email_validation.settings');
    $blacklist_domains = explode(',', $config->get('blacklist.domains'));
    foreach ($blacklist_domains as $domain) {
      if (trim(strtolower($domain)) == trim(strtolower($email_domain))) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Extracts the domain name from an email address.
   *
   * @param string $email
   *   The email address.
   *
   * @return string
   *   The domain name.
   */
  public function extractDomain($email) {
    // Split the email address at the "@" symbol.
    $parts = explode('@', $email);

    // Return the domain part if it exists.
    return $parts[1] ?? '';
  }

}
